﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;

public class EnemyBehaviourScript : MonoBehaviour
{
    public NavMeshAgent navAgent;
    public Transform navAgentTarget;
    public float enemySpeed;

    public SphereCollider enemyTrigger;
    public float playerDetectionRadius;
    public float timeBeforeGivingUp; //Temps pris par l'ennemi avant de reprendre le roam si il ne detecte pas le joueur
    float detectionTimer;
    public bool playerDetected;
    public bool interactWithPlayer;
    public bool isRoaming;

    public bool isAttacking;
    public Transform attackPoint;
    public LayerMask attackLayer;
    public int enemyDamage;
    public float attackCD;

    public Animator anim;

    public enum EnemyTypes { Attacking, Fleeing }
    public EnemyTypes enemyType;

    [Header("Debug")]
    public Transform DebugPlayerTarget;

    public HealthSystem hs;

    // Start is called before the first frame update
    void Start()
    {
        navAgent = GetComponent<NavMeshAgent>();
        hs = GetComponent<HealthSystem>();
        navAgent.speed = enemySpeed;
        enemyTrigger.radius = playerDetectionRadius;
    }

    private void Update()
    {
        if (hs.dead) return;

        if (detectionTimer < timeBeforeGivingUp && !playerDetected)
        {
            detectionTimer += Time.deltaTime;
        }
        else if (detectionTimer >= timeBeforeGivingUp)
            interactWithPlayer = false;

        if (interactWithPlayer)
        {
            switch (enemyType)
            {
                case EnemyTypes.Attacking:
                    Attack();
                    break;
                case EnemyTypes.Fleeing:
                    Flee();
                    break;
                default:
                    break;
            }
        }
        else if (!isRoaming)
        {
            StartCoroutine(Roam());
        }

        anim.SetFloat("Speed", navAgent.velocity.magnitude);
        

    }

    private void OnTriggerStay(Collider other)
    {
        if (other.CompareTag("P"))
        {
            if(!playerDetected)
            {
                playerDetected = true;
                detectionTimer = 0;
                interactWithPlayer = true;
                navAgentTarget = other.transform;
                isRoaming = false;
            }
        }
    }

    void Flee()
    {
        Vector3 targetDirection = transform.position - navAgentTarget.position;
        targetDirection.y = transform.position.y;
        navAgent.SetDestination(transform.position + Vector3.Normalize(targetDirection));
    }

    void Attack()
    {
        navAgent.SetDestination(navAgentTarget.position);

        float playerDistance = Vector3.Distance(transform.position, navAgentTarget.position);

        if (playerDistance < Vector3.Distance(transform.position,attackPoint.position))
        {
            navAgent.isStopped = true;

            if (!isAttacking)
            {
            isAttacking = true;
                //lancer l'anim, l'anim lance Hit() au moment de toucher le joueur
                anim.SetTrigger("Attack");
            StartCoroutine(AttackingCD());
            }
        }
        else
            navAgent.isStopped = false;
    }

    public void Hit()
    {
        Collider[] hit = Physics.OverlapSphere(attackPoint.position, 0.5f, attackLayer);
        foreach (Collider c in hit)
        {
            if (c.CompareTag("P"))
            {               
                c.GetComponent<HealthSystem>().Hurt(enemyDamage, name);
            }
        }
    }

    private void OnTriggerExit(Collider other)
    {
        if (other.CompareTag("P"))
        {           
            playerDetected = false;
        }
    }

    IEnumerator AttackingCD()
    {
        yield return new WaitForSeconds(attackCD);
        //Debug.Log("Hitting!");
        ///Hit();

        isAttacking = false;
    }

    IEnumerator Roam()
    {
        isRoaming = true;

        while (!interactWithPlayer)
        {
            yield return new WaitForSeconds(Random.Range(2, 8f));

            Vector3 ranpos = new Vector3(Random.Range(transform.position.x - 7, transform.position.x + 7),
                                         0,
                                         Random.Range(transform.position.z - 7, transform.position.z + 7));
            navAgent.SetDestination(ranpos);

            Debug.Log("Roaming!");

            if (interactWithPlayer)
            {
                break;
            }
        }
    }

    private void OnDrawGizmos()
    {
        Gizmos.color = Color.red;
        Gizmos.DrawWireSphere(transform.position, playerDetectionRadius);
        Gizmos.color = Color.magenta;
        Gizmos.DrawWireSphere(attackPoint.position, 0.5f);
    }
}
