﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

[RequireComponent(typeof(CharacterController))]
public class PlayerMouvement : MonoBehaviour
{
    public float speed;
    public float turnSpeed = 100f;
    public Animator anim;
    public bool canMove;

    [Header("Gravity / Jump")]
    public float gravity;
    public LayerMask groundLayer;
    public float groundDistance;
    public float jumpHeight;

    bool isGrounded,hasjumped;
    CharacterController controller;
    Camera playercam;
    Vector2 input;
    Vector3 velocity,moveDir;

    // Start is called before the first frame update
    void Start()
    {
        controller = GetComponent<CharacterController>();
        playercam = Camera.main;
        canMove = true;

    }

    // Update is called once per frame
    void Update()
    {
        GetInput();

        CalculateGravity();

        anim.SetFloat("Speed", input.magnitude);


        //setting moveVector
        moveDir = playercam.transform.forward * input.y + playercam.transform.right * input.x;

        if (isGrounded)
        {
            RaycastHit hit;
            Physics.Raycast(transform.position, Vector3.down * groundDistance, out hit, groundLayer);
            moveDir = Vector3.ProjectOnPlane(moveDir, hit.normal);
        }
        else
        {
            moveDir = Vector3.ProjectOnPlane(moveDir, Vector3.up);
        }

        //rotatePlayer if input

        if (moveDir != Vector3.zero)
        {
            float angle = playercam.transform.rotation.eulerAngles.y + Mathf.Atan2(input.x, input.y) * Mathf.Rad2Deg;
            Quaternion targetangle = Quaternion.AngleAxis(angle, Vector3.up);
            transform.rotation = Quaternion.RotateTowards(transform.rotation,targetangle,turnSpeed* Time.deltaTime);
        }

        /*
        //Jump
        if(Input.GetButton("Jump") && isGrounded && !hasjumped)
        {
            velocity.y = Mathf.Sqrt(jumpHeight * -2 * gravity);
            hasjumped = true;
        }
        */
    }

    private void LateUpdate()
    {
        if (canMove)
            controller.Move(moveDir * speed * Time.deltaTime);

        controller.Move(velocity * Time.deltaTime);
    }

    /// <summary>
    /// Get the Player's input and normalise it if it's greater than 1
    /// </summary>
    void GetInput()
    {
        float h = Input.GetAxis("Horizontal");
        float v = Input.GetAxis("Vertical");

        input = new Vector2(h, v);

        if (input.magnitude > 1) input.Normalize();
    }

    void CalculateGravity()
    {
        isGrounded = Physics.CheckSphere(transform.position, groundDistance, groundLayer);

        if (isGrounded)
        {
            velocity.y = -2f;
            if (hasjumped)
            {
                hasjumped = false;
            }
        }
            
        velocity.y += gravity * Time.deltaTime;
    }

    private void OnDrawGizmosSelected()
    {
        Gizmos.color = Color.red;
        Gizmos.DrawWireSphere(transform.position, groundDistance);
    }
}
